@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -64,21 +64,60 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /*************          Internal functions         *************/
 /***************************************************************/
 #define SET_NUM_OP(CTYPE) {                                \
-    CTYPE a=*(CTYPE *)(data->array); if(a>0) return a;    }
+    CTYPE a=*(CTYPE *)(numpop->array); if(a>0) return a;    }
 
 static size_t
-pop_number_of_operands(struct arithmeticparams *p, gal_data_t *data,
-                       char *token_string)
+pop_number_of_operands(struct arithmeticparams *p, int op, char *token_string,
+                       gal_data_t **params)
 {
+  char *cstring="first";
+  size_t c, numparams=0;
+  gal_data_t *tmp, *numpop;
+
+  /* See if this operator needs any parameters. If so, pop them. */
+  switch(op)
+    {
+    case GAL_ARITHMETIC_OP_SIGCLIP_STD:
+    case GAL_ARITHMETIC_OP_SIGCLIP_MEAN:
+    case GAL_ARITHMETIC_OP_SIGCLIP_MEDIAN:
+    case GAL_ARITHMETIC_OP_SIGCLIP_NUMBER:
+      numparams=2;
+      break;
+    }
+
+  /* If any parameters should be read, read them. */
+  *params=NULL;
+  for(c=0;c<numparams;++c)
+    {
+      /* If it only has one element, save it as floating point and add it
+         to the list. */
+      tmp=operands_pop(p, token_string);
+      if(tmp->size>1)
+        error(EXIT_FAILURE, 0, "the %s popped operand of the \"%s\" "
+              "operator must be a single number", cstring, token_string);
+      tmp=gal_data_copy_to_new_type_free(tmp, GAL_TYPE_FLOAT32);
+      gal_list_data_add(params, tmp);
+
+      /* A small sanity check (none of the parameters for sigma-clipping
+         can be negative.. */
+      if( ((float *)(tmp->array))[0]<=0.0 )
+        error(EXIT_FAILURE, 0, "the %s popped operand of the \"%s\" "
+              "operator cannot be negative", cstring, token_string);
+
+      /* Increment the counter string. */
+      cstring=c?"third":"second";
+    }
+
   /* Check if its a number. */
-  if(data->size>1)
-    error(EXIT_FAILURE, 0, "the first popped operand to the \"%s\" "
-          "operator must be a number, not an array", token_string);
+  numpop=operands_pop(p, token_string);
+  if(numpop->size>1)
+    error(EXIT_FAILURE, 0, "the %s popped operand of the \"%s\" "
+          "operator (number of input datasets) must be a number, not an "
+          "array", cstring, token_string);
 
   /* Check its type and return the value. */
-  switch(data->type)
+  switch(numpop->type)
     {
-
     /* For the integer types, if they are unsigned, then just pass their
        value, if they are signed, you have to make sure they are zero or
        positive. */
@@ -94,18 +133,20 @@ pop_number_of_operands(struct arithmeticparams *p, gal_data_t *data,
     /* Floating point numbers are not acceptable in this context. */
     case GAL_TYPE_FLOAT32:
     case GAL_TYPE_FLOAT64:
-      error(EXIT_FAILURE, 0, "the first popped operand to the \"%s\" "
-            "operator must be an integer type", token_string);
+      error(EXIT_FAILURE, 0, "the %s popped operand of the \"%s\" "
+            "operator (number of input datasets) must be an integer type",
+            cstring, token_string);
 
     default:
       error(EXIT_FAILURE, 0, "%s: type code %d not recognized",
-            __func__, data->type);
+            __func__, numpop->type);
     }
 
   /* If control reaches here, then the number must have been a negative
      value, so print an error. */
-  error(EXIT_FAILURE, 0, "the first popped operand to the \"%s\" operator "
-        "cannot be zero or a negative number", token_string);
+  error(EXIT_FAILURE, 0, "the %s popped operand of the \"%s\" operator "
+        "cannot be zero or a negative number", cstring,
+        token_string);
   return 0;
 }
 
@@ -390,7 +431,7 @@ wrapper_for_filter(struct arithmeticparams *p, char *token, int operator)
                   "along dimension %zu is a float", ndim-i);
 
           /* Make sure it isn't negative. */
-          comp=gal_arithmetic(GAL_ARITHMETIC_OP_GT, 0, tmp, zero);
+          comp=gal_arithmetic(GAL_ARITHMETIC_OP_GT, 1, 0, tmp, zero);
           if( *(uint8_t *)(comp->array) == 0 )
             error(EXIT_FAILURE, 0, "lengths of filter along dimensions "
                   "must be positive. The given length in dimension %zu"
@@ -687,8 +728,9 @@ arithmetic_interpolate(struct arithmeticparams *p, char *token)
   num_int = *((int32_t *)(num->array));
 
   /* Call the interpolation function. */
-  interpolated=gal_interpolate_close_neighbors(in, NULL, num_int,
-                                               p->cp.numthreads, 1, 0);
+  interpolated=gal_interpolate_close_neighbors(in, NULL, p->cp.interpmetric,
+                                               num_int, p->cp.numthreads,
+                                               1, 0);
 
   /* Clean up and push the interpolated array onto the stack. */
   gal_data_free(in);
@@ -781,13 +823,17 @@ arithmetic_collapse(struct arithmeticparams *p, char *token, int operator)
     }
 
 
-  /* We'll also need to correct the size of the reference dataset. We'll
-     use `memcpy' to write the new `dsize' values into the old ones. The
-     dimensions have decreased, so we won't be writing outside of allocated
-     space that `p->refdata.dsize' points to. */
-  p->refdata.ndim -= 1;
-  memcpy( p->refdata.dsize, collapsed->dsize,
-           p->refdata.ndim * (sizeof *p->refdata.dsize) );
+  /* We'll also need to correct the size of the reference dataset if it
+     hasn't been corrected yet. We'll use `memcpy' to write the new `dsize'
+     values into the old ones. The dimensions have decreased, so we won't
+     be writing outside of allocated space that `p->refdata.dsize' points
+     to. */
+  if( p->refdata.ndim != collapsed->ndim )
+    {
+      p->refdata.ndim -= 1;
+      memcpy( p->refdata.dsize, collapsed->dsize,
+              p->refdata.ndim * (sizeof *p->refdata.dsize) );
+    }
 
 
   /* Clean up and add the collapsed dataset to the top of the operands. */
@@ -800,6 +846,37 @@ arithmetic_collapse(struct arithmeticparams *p, char *token, int operator)
 
 
 
+void
+arithmetic_tofile(struct arithmeticparams *p, char *token, int freeflag)
+{
+  /* Pop the top dataset. */
+  gal_data_t *popped = operands_pop(p, token);
+  char *filename = &token[ freeflag
+                           ? OPERATOR_PREFIX_LENGTH_TOFILEFREE
+                           : OPERATOR_PREFIX_LENGTH_TOFILE     ];
+
+  /* Save it to a file. */
+  popped->wcs=p->refdata.wcs;
+  if(popped->ndim==1 && p->onedasimage==0)
+    gal_table_write(popped, NULL, p->cp.tableformat, filename,
+                    "ARITHMETIC", 0);
+  else
+    gal_fits_img_write(popped, filename, NULL, PROGRAM_NAME);
+  if(!p->cp.quiet)
+    printf(" - Write: %s\n", filename);
+
+  /* Reset the WCS to NULL and put it back on the stack. */
+  popped->wcs=NULL;
+  if(freeflag)
+    gal_data_free(popped);
+  else
+    operands_add(p, NULL, popped);
+}
+
+
+
+
+
 
 
 
@@ -828,10 +905,10 @@ void
 reversepolish(struct arithmeticparams *p)
 {
   int op=0, nop=0;
-  char *filename, *hdu;
   unsigned int numop, i;
   gal_list_str_t *token;
-  gal_data_t *d1=NULL, *d2=NULL, *d3=NULL;
+  gal_data_t *d1, *d2, *d3;
+  char *hdu, *filename, *printnum;
   int flags = ( GAL_ARITHMETIC_INPLACE | GAL_ARITHMETIC_FREE
                 | GAL_ARITHMETIC_NUMOK );
 
@@ -844,20 +921,27 @@ reversepolish(struct arithmeticparams *p)
   /* Go over each input token and do the work. */
   for(token=p->tokens;token!=NULL;token=token->next)
     {
-      /* If we have a name or number, then add it to the operands linked
-         list. Otherwise, pull out two members and do the specified
-         operation on them. */
-      if( gal_array_name_recognized(token->v)
+      /* The `tofile-' operator's string can end in a `.fits', similar to a
+         FITS file input file. So, it needs to be checked before checking
+         for a filename. If we have a name or number, then add it to the
+         operands linked list. Otherwise, pull out two members and do the
+         specified operation on them. */
+      if( !strncmp(OPERATOR_PREFIX_TOFILE, token->v,
+                   OPERATOR_PREFIX_LENGTH_TOFILE) )
+        arithmetic_tofile(p, token->v, 0);
+      else if( !strncmp(OPERATOR_PREFIX_TOFILEFREE, token->v,
+                   OPERATOR_PREFIX_LENGTH_TOFILE) )
+        arithmetic_tofile(p, token->v, 1);
+      else if( !strncmp(token->v, OPERATOR_PREFIX_SET,
+                        OPERATOR_PREFIX_LENGTH_SET) )
+        operands_set_name(p, token->v);
+      else if( gal_array_name_recognized(token->v)
           || operands_is_name(p, token->v) )
         operands_add(p, token->v, NULL);
       else if( (d1=gal_data_copy_string_to_number(token->v)) )
         operands_add(p, NULL, d1);
-      else if( !strncmp(token->v, SET_OPERATOR_PREFIX,
-                        SET_OPERATOR_PREFIX_LENGTH) )
-        operands_set_name(p, token->v);
       else
         {
-
           /* Order is the same as in the manual. */
           /* Simple arithmetic operators. */
           if      (!strcmp(token->v, "+" ))
@@ -888,8 +972,8 @@ reversepolish(struct arithmeticparams *p)
             { op=GAL_ARITHMETIC_OP_MINVAL;            nop=1;  }
           else if (!strcmp(token->v, "maxvalue"))
             { op=GAL_ARITHMETIC_OP_MAXVAL;            nop=1;  }
-          else if (!strcmp(token->v, "numvalue"))
-            { op=GAL_ARITHMETIC_OP_NUMVAL;            nop=1;  }
+          else if (!strcmp(token->v, "numbervalue"))
+            { op=GAL_ARITHMETIC_OP_NUMBERVAL;         nop=1;  }
           else if (!strcmp(token->v, "sumvalue"))
             { op=GAL_ARITHMETIC_OP_SUMVAL;            nop=1;  }
           else if (!strcmp(token->v, "meanvalue"))
@@ -902,8 +986,8 @@ reversepolish(struct arithmeticparams *p)
             { op=GAL_ARITHMETIC_OP_MIN;               nop=-1; }
           else if (!strcmp(token->v, "max"))
             { op=GAL_ARITHMETIC_OP_MAX;               nop=-1; }
-          else if (!strcmp(token->v, "num"))
-            { op=GAL_ARITHMETIC_OP_NUM;               nop=-1; }
+          else if (!strcmp(token->v, "number"))
+            { op=GAL_ARITHMETIC_OP_NUMBER;            nop=-1; }
           else if (!strcmp(token->v, "sum"))
             { op=GAL_ARITHMETIC_OP_SUM;               nop=-1; }
           else if (!strcmp(token->v, "mean"))
@@ -912,6 +996,14 @@ reversepolish(struct arithmeticparams *p)
             { op=GAL_ARITHMETIC_OP_STD;               nop=-1; }
           else if (!strcmp(token->v, "median"))
             { op=GAL_ARITHMETIC_OP_MEDIAN;            nop=-1; }
+          else if (!strcmp(token->v, "sigclip-number"))
+            { op=GAL_ARITHMETIC_OP_SIGCLIP_NUMBER;    nop=-1; }
+          else if (!strcmp(token->v, "sigclip-mean"))
+            { op=GAL_ARITHMETIC_OP_SIGCLIP_MEAN;      nop=-1; }
+          else if (!strcmp(token->v, "sigclip-median"))
+            { op=GAL_ARITHMETIC_OP_SIGCLIP_MEDIAN;    nop=-1; }
+          else if (!strcmp(token->v, "sigclip-std"))
+            { op=GAL_ARITHMETIC_OP_SIGCLIP_STD;       nop=-1; }
 
           /* Conditional operators. */
           else if (!strcmp(token->v, "lt" ))
@@ -1004,14 +1096,15 @@ reversepolish(struct arithmeticparams *p)
             { op=ARITHMETIC_OP_COLLAPSE_MEAN;         nop=0; }
           else if (!strcmp(token->v, "collapse-number"))
             { op=ARITHMETIC_OP_COLLAPSE_NUMBER;       nop=0; }
-
-
-          /* Finished checks with known operators */
           else
             error(EXIT_FAILURE, 0, "the argument \"%s\" could not be "
                   "interpretted as a file name, named dataset, number, or "
                   "operator", token->v);
 
+          /* Initialize all the operand pointers (they may be remaining
+             from previous operators and we don't want them to cause
+             confusion. */
+          d1 = d2 = d3 = NULL;
 
           /* See if the arithmetic library must be called or not. */
           if(nop)
@@ -1040,13 +1133,12 @@ reversepolish(struct arithmeticparams *p)
 
                 case -1:
                   /* This case is when the number of operands is itself an
-                     operand. So the first popped operand must be an
+                     operand. So except for sigma-clipping (that has other
+                     parameters), the first popped operand must be an
                      integer number, we will use that to construct a linked
                      list of any number of operands within the single `d1'
                      pointer. */
-                  d1=NULL;
-                  numop=pop_number_of_operands(p, operands_pop(p, token->v),
-                                               token->v);
+                  numop=pop_number_of_operands(p, op, token->v, &d2);
                   for(i=0;i<numop;++i)
                     gal_list_data_add(&d1, operands_pop(p, token->v));
                   break;
@@ -1056,13 +1148,13 @@ reversepolish(struct arithmeticparams *p)
                         "operand(s)", token->v, nop);
                 }
 
-
               /* Run the arithmetic operation. Note that `gal_arithmetic'
                  is a variable argument function (like printf). So the
                  number of arguments it uses depend on the operator. So
                  when the operator doesn't need three operands, the extra
                  arguments will be ignored. */
-              operands_add(p, NULL, gal_arithmetic(op, flags, d1, d2, d3));
+              operands_add(p, NULL, gal_arithmetic(op, p->cp.numthreads,
+                                                   flags, d1, d2, d3));
             }
 
           /* No need to call the arithmetic library, call the proper
@@ -1146,9 +1238,9 @@ reversepolish(struct arithmeticparams *p)
       filename=p->operands->filename;
       if( gal_fits_name_is_fits(filename) )
         {
+          /* Read the data, note that the WCS has already been set. */
           p->operands->data=gal_array_read_one_ch(filename, hdu, NULL,
                                                   p->cp.minmapsize);
-          p->refdata.wcs=gal_wcs_read(filename, hdu, 0, 0, &p->refdata.nwcs);
           if(!p->cp.quiet) printf(" - %s (hdu %s) is read.\n", filename, hdu);
         }
       else
@@ -1164,11 +1256,12 @@ reversepolish(struct arithmeticparams *p)
   d1=p->operands->data;
   if(d1->size==1)
     {
-      /* To simplify the printing process, we will first change it to
-         double, then use printf's `%g' to print it, so integers will be
-         printed as an integer.  */
-      d2=gal_data_copy_to_new_type(d1, GAL_TYPE_FLOAT64);
-      printf("%g\n", *(double *)d2->array);
+      /* Make the string to print the number. */
+      printnum=gal_type_to_string(d1->array, d1->type, 0);
+      printf("%s\n", printnum);
+
+      /* Clean up. */
+      free(printnum);
       if(d2!=d1) gal_data_free(d2);
     }
   else
@@ -1182,7 +1275,7 @@ reversepolish(struct arithmeticparams *p)
       else
         gal_fits_img_write(d1, p->cp.output, NULL, PROGRAM_NAME);
       if(!p->cp.quiet)
-        printf(" - Output written to %s\n", p->cp.output);
+        printf(" - Write (final): %s\n", p->cp.output);
     }
 
 
