@@ -5,7 +5,7 @@ ConvertType is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2018, Free Software Foundation, Inc.
+Copyright (C) 2016-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -226,6 +226,85 @@ parse_opt(int key, char *arg, struct argp_state *state)
 /**************************************************************/
 /***************       Sanity Check         *******************/
 /**************************************************************/
+static void
+ui_colormap_sanity_check(struct converttparams *p)
+{
+  char **strarr;
+  float *farray;
+  size_t nparams=0;
+  int ccode=COLOR_INVALID;
+
+  /* See how many parameters are necessary. */
+  strarr=p->colormap->array;
+  if     ( !strcmp(strarr[0],"hsv" )) { ccode=COLOR_HSV;  nparams=2; }
+  else if( !strcmp(strarr[0],"sls" )) { ccode=COLOR_SLS;  nparams=0; }
+  else if( !strcmp(strarr[0],"gray") || !strcmp(strarr[0],"grey"))
+                                      { ccode=COLOR_GRAY; nparams=0; }
+  else
+    error(EXIT_FAILURE, 0, "`%s' not recognized as a color-space given "
+          "to `--monotocolor'", strarr[0]);
+  p->colormap->status=ccode;
+
+  /* Check if the proper number of parameters are given for this color
+     space. Note that the actual colorspace name is the first element in
+     `monotocolor'. */
+  if(p->colormap->size!=1 && p->colormap->size != (nparams+1) )
+    error(EXIT_FAILURE, 0, "%zu parameters given to `--monotocolor' for "
+          "the `%s' color space (which needs %zu)",
+          p->colormap->size-1, strarr[0], nparams);
+
+  /* Allocate the necessary space for the parameters (when
+     necessary). */
+  if(nparams>0)
+    {
+      /* If no parameters were given, put the full range. */
+      if(p->colormap->size==1)
+        {
+          p->colormap->next=gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1,
+                                            &nparams, NULL, 0,
+                                            p->cp.minmapsize, NULL,NULL,NULL);
+          farray=p->colormap->next->array;
+          switch(p->colormap->status)
+            {
+            case COLOR_HSV: farray[0]=0; farray[1]=360; break;
+            default:
+              error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at "
+                    "%s to fix the problem. The value `%d' is not "
+                    "recognized for a color space that needs default "
+                    "parameters", __func__, PACKAGE_BUGREPORT,
+                    p->colormap->status);
+            }
+        }
+      else
+        {
+          /* Increment the array pointer (temporarily) so we can read
+             the rest of the parameters as float32. Note that we can't
+             use `+=' because it is a `void *' pointer. We'll have to
+             use `strarry', because its type is defined. */
+          p->colormap->size-=1;
+          p->colormap->array = strarr + 1;
+          p->colormap->next=gal_data_copy_to_new_type(p->colormap,
+                                                       GAL_TYPE_FLOAT32);
+          p->colormap->array = strarr;
+          p->colormap->size+=1;
+
+          /* For a check
+             {
+               size_t i;
+               farray=p->monotocolor->next->array;
+               for(i=0;i<p->monotocolor->next->size;++i)
+               printf("%f\n", farray[i]);
+               exit(1);
+             }
+          */
+        }
+    }
+}
+
+
+
+
+
 /* Read and check ONLY the options. When arguments are involved, do the
    check in `ui_check_options_and_arguments'. */
 static void
@@ -253,7 +332,7 @@ ui_read_check_only_options(struct converttparams *p)
 
   if(p->fluxhighstr && p->fluxlowstr)
     {
-      cond=gal_arithmetic(GAL_ARITHMETIC_OP_GT, GAL_ARITHMETIC_NUMOK,
+      cond=gal_arithmetic(GAL_ARITHMETIC_OP_GT, 1, GAL_ARITHMETIC_NUMOK,
                           p->fluxhigh, p->fluxlow);
 
       if( *((unsigned char *)cond->array) == 0 )
@@ -262,6 +341,10 @@ ui_read_check_only_options(struct converttparams *p)
 
       gal_data_free(cond);
     }
+
+  /* Check the colormap. */
+  if(p->colormap)
+    ui_colormap_sanity_check(p);
 }
 
 
@@ -271,14 +354,11 @@ ui_read_check_only_options(struct converttparams *p)
 static void
 ui_check_options_and_arguments(struct converttparams *p)
 {
-  /* Check if there was any inputs. */
-  if(p->inputnames==NULL)
-    error(EXIT_FAILURE, 0, "no input files given");
-
-  /* Reverse the `inputnames' linked list, note that the `hdu' linked list
-     was reversed during option parsing.*/
+  /* Reverse the `inputnames' linked list if it was given (recall that we
+     also accept input from the standard input). Note that the `hdu' linked
+     list was reversed during option parsing, so we don't need to do it
+     here any more. */
   gal_list_str_reverse(&p->inputnames);
-
 }
 
 
@@ -396,11 +476,27 @@ ui_make_channels_ll(struct converttparams *p)
 {
   char *hdu=NULL;
   gal_data_t *data;
-  gal_list_str_t *name;
   size_t dsize=0, dirnum;
+  gal_list_str_t *name, *lines;
+
+  /* Initialize the counting of channels. */
+  p->numch=0;
+
+  /* If any standard input is provided, we want to process that first. Note
+     that since other input arguments are also allowed (as other channels),
+     we'll need to process the standard input independently first, then go
+     onto the possible list of other files.*/
+  lines=gal_txt_stdin_read(p->cp.stdintimeout);
+  if(lines)
+    {
+      data=gal_txt_image_read(NULL, lines, p->cp.minmapsize);
+      gal_list_data_add(&p->chll, data);
+      gal_list_str_free(lines, 1);
+      ++p->numch;
+    }
+
 
   /* Go through the input files and add the channel(s). */
-  p->numch=0;
   for(name=p->inputnames; name!=NULL; name=name->next)
     {
       /* Check if p->numch has not exceeded 4. */
@@ -492,12 +588,16 @@ ui_make_channels_ll(struct converttparams *p)
       /* Text: */
       else
         {
-          data=gal_txt_image_read(name->v, p->cp.minmapsize);
+          data=gal_txt_image_read(name->v, NULL, p->cp.minmapsize);
           gal_list_data_add(&p->chll, data);
           ++p->numch;
         }
     }
 
+  /* If there weren't any channels, abort with an error. */
+  if(p->numch==0)
+    error(EXIT_FAILURE, 0, "%s",
+          gal_options_stdin_error(p->cp.stdintimeout, 0, "input"));
 
   /* Reverse the list of channels into the input order. */
   gal_list_data_reverse(&p->chll);
@@ -529,6 +629,27 @@ ui_prepare_input_channels(struct converttparams *p)
           "contain more than one color channel", p->numch);
 
 
+  /* If there are multiple color channels, then ignore the monotocolor
+     option if it is given. But if there is only one, make sure that the
+     `colormap' option is actually given.*/
+  if( p->numch==1 )
+    {
+      if( p->colormap==NULL )
+        error(EXIT_FAILURE, 0, "no colormap! When there is only one input "
+              "channel, it is necessary to specify a color map. For "
+              "example `gray', `hsv', or `sls'.\n\n"
+              "For more on ConvertType's color mapping, see the "
+              "description under `--colormap' in the Gnuastro book:\n\n"
+              "   $ info astconvertt");
+    }
+  else if( p->numch>1 && p->colormap )
+    {
+      if(p->colormap->next) gal_data_free(p->colormap->next);
+      gal_data_free(p->colormap);
+      p->colormap=NULL;
+    }
+
+
   /* Go over the channels and make the proper checks/corrections. We won't
      be checking blank channels here, recall that blank channels had a
      dimension of zero. */
@@ -616,7 +737,7 @@ void
 ui_add_dot_use_automatic_output(struct converttparams *p)
 {
   gal_list_str_t *stll;
-  char *tmp, *firstname="output.txt", *suffix=p->cp.output;
+  char *tmp, *firstname="converttype.txt", *suffix=p->cp.output;
 
   /* Find the first non-blank file name in the input(s). */
   for(stll=p->inputnames; stll!=NULL; stll=stll->next)
@@ -686,6 +807,9 @@ ui_set_output(struct converttparams *p)
   /* EPS */
   else if(gal_eps_name_is_eps(cp->output))
     {
+      if(p->borderwidth==0 && p->widthincm==0)
+        error(EXIT_FAILURE, 0, "at least one of `--widthincm' (`-u'), or "
+              "`--borderwidth (`-b') options are necessary for an EPS output");
       p->outformat=OUT_FORMAT_EPS;
       if( gal_eps_suffix_is_eps(cp->output) )
         ui_add_dot_use_automatic_output(p);
@@ -694,6 +818,9 @@ ui_set_output(struct converttparams *p)
   /* PDF */
   else if(gal_pdf_name_is_pdf(cp->output))
     {
+      if(p->borderwidth==0 && p->widthincm==0)
+        error(EXIT_FAILURE, 0, "at least one of `--widthincm' (`-u'), or "
+              "`--borderwidth (`-b') options are necessary for a PDF output");
       p->outformat=OUT_FORMAT_PDF;
       if( gal_pdf_suffix_is_pdf(cp->output) )
         ui_add_dot_use_automatic_output(p);
@@ -857,7 +984,11 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct converttparams *p)
 void
 ui_free_report(struct converttparams *p)
 {
-  /* Free the allocated spaces */
+  if(p->colormap)
+    {
+      if(p->colormap->next) gal_data_free(p->colormap->next);
+      gal_data_free(p->colormap);
+    }
   gal_data_free(p->fluxlow);
   gal_data_free(p->fluxhigh);
   gal_list_str_free(p->hdus, 1);
