@@ -5,7 +5,7 @@ Crop is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -101,6 +101,9 @@ wcsmode_check_prepare(struct cropparams *p, struct inputimgs *img)
      small differences might exist in the pixel scale, so break out with an
      error only if the pixel scales are more different than 1e-6. */
   pixscale=gal_wcs_pixel_scale(wcs);
+  if(pixscale==NULL)
+    error(EXIT_FAILURE, 0, "the pixel scale couldn't be deduced from the "
+          "WCS");
   if( fabs(pixscale[0]-pixscale[1])/pixscale[0] > 1e-6 )
     error(EXIT_FAILURE, 0, "%s: HDU %s: The pixel scale along "
           "the two image axises is not the same. The first axis "
