@@ -3,9 +3,9 @@ Fits - View and manipulate FITS extensions and/or headers.
 Fits is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016, Free Software Foundation, Inc.
+Copyright (C) 2016-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -31,7 +31,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /* Array of acceptable options. */
 struct argp_option program_options[] =
   {
-
+    /* Input options */
     {
       0, 0, 0, 0,
       "HDUs (extensions):",
@@ -76,6 +76,21 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "numhdus",
+      UI_KEY_NUMHDUS,
+      0,
+      0,
+      "Print number of HDUs in the given FITS file.",
+      UI_GROUP_EXTENSION,
+      &p->numhdus,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+
+
 
 
 
@@ -157,7 +172,7 @@ struct argp_option program_options[] =
       "Add HISTORY keyword, any length is ok.",
       UI_GROUP_KEYWORD,
       &p->history,
-      GAL_TYPE_STRING,
+      GAL_TYPE_STRLL,
       GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
@@ -170,7 +185,7 @@ struct argp_option program_options[] =
       "Add COMMENT keyword, any length is ok.",
       UI_GROUP_KEYWORD,
       &p->comment,
-      GAL_TYPE_STRING,
+      GAL_TYPE_STRLL,
       GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
@@ -188,6 +203,19 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "verify",
+      UI_KEY_VERIFY,
+      0,
+      0,
+      "Verify the CHECKSUM and DATASUM keywords.",
+      UI_GROUP_KEYWORD,
+      &p->verify,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "printallkeys",
       UI_KEY_PRINTALLKEYS,
@@ -201,9 +229,55 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "copykeys",
+      UI_KEY_COPYKEYS,
+      "INT:INT",
+      0,
+      "Range of keywords to copy to output HDU.",
+      UI_GROUP_KEYWORD,
+      &p->copykeys,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "datetosec",
+      UI_KEY_DATETOSEC,
+      "STR",
+      0,
+      "FITS date to sec from 1970/01/01T00:00:00",
+      UI_GROUP_KEYWORD,
+      &p->datetosec,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
 
 
 
+
+
+    /* Output options. */
+    {
+      "outhdu",
+      UI_KEY_OUTHDU,
+      "STR",
+      0,
+      "HDU/extension in output for --copykeys.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->outhdu,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+
+
+
+    /* Operating mode options. */
     {
       "quitonerror",
       UI_KEY_QUITONERROR,
