@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2018, Free Software Foundation, Inc.
+Copyright (C) 2016-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -334,13 +334,13 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET
     },
     {
-      "checkupperlimit",
-      UI_KEY_CHECKUPPERLIMIT,
+      "checkuplim",
+      UI_KEY_CHECKUPLIM,
       "INT[,INT]",
       0,
       "Check random distribution for one label.",
       UI_GROUP_UPPERLIMIT,
-      &p->checkupperlimit,
+      &p->checkuplim,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_GT_0,
       GAL_OPTIONS_NOT_MANDATORY,
@@ -482,6 +482,62 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET,
       ui_column_codes_ll
     },
+    {
+      "minx",
+      UI_KEY_MINX,
+      0,
+      0,
+      "Minimum first FITS axis position.",
+      UI_GROUP_COLUMNS_POSITION_PIXEL,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "maxx",
+      UI_KEY_MAXX,
+      0,
+      0,
+      "Maximum first FITS axis position.",
+      UI_GROUP_COLUMNS_POSITION_PIXEL,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "miny",
+      UI_KEY_MINY,
+      0,
+      0,
+      "Minimum second FITS axis position.",
+      UI_GROUP_COLUMNS_POSITION_PIXEL,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
+    {
+      "maxy",
+      UI_KEY_MAXY,
+      0,
+      0,
+      "Maximum second FITS axis position.",
+      UI_GROUP_COLUMNS_POSITION_PIXEL,
+      0,
+      GAL_TYPE_INVALID,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_column_codes_ll
+    },
     {
       "clumpsx",
       UI_KEY_CLUMPSX,
@@ -743,8 +799,8 @@ struct argp_option program_options[] =
       ui_column_codes_ll
     },
     {
-      "noriverbrightness",
-      UI_KEY_NORIVERBRIGHTNESS,
+      "brightnessnoriver",
+      UI_KEY_BRIGHTNESSNORIVER,
       0,
       0,
       "Sky (not river) subtracted clump brightness.",
@@ -957,7 +1013,7 @@ struct argp_option program_options[] =
       UI_KEY_SKY,
       0,
       0,
-      "Average Sky value under this clump or object.",
+      "Sky value (per pixel).",
       UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
@@ -971,7 +1027,7 @@ struct argp_option program_options[] =
       UI_KEY_STD,
       0,
       0,
-      "Average Sky standard deviation under label.",
+      "Sky standard deviation (per pixel).",
       UI_GROUP_COLUMNS_BRIGHTNESS,
       0,
       GAL_TYPE_INVALID,
