@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -633,7 +633,7 @@ ui_read_cols(struct mkprofparams *p)
     gal_checkset_allocate_copy("standard-input", &p->catname);
 
   /* Set the number of objects. */
-  p->num=cols->size;
+  p->num = cols ? cols->size : 0;
 
   /* Put each column's data in the respective internal array. */
   while(cols!=NULL)
@@ -792,6 +792,25 @@ ui_read_cols(struct mkprofparams *p)
           gal_data_free(corrtype);
         }
     }
+
+  /* Multi-column sanity checks. */
+  counter=0;
+  if( !p->cp.quiet && (p->mforflatpix || p->mcolisbrightness) )
+    for(i=0;i<p->num;++i)
+      if( p->m[i]==0.0 && ( p->f[i]==PROFILE_POINT
+                            || p->f[i]==PROFILE_FLAT
+                            || p->f[i]==PROFILE_CIRCUMFERENCE ) )
+        {
+          error(0, 0, "WARNING: atleast one single-valued profile (point, "
+                "flat, or circumference profiles) has a magnitude column "
+                "value of 0.0 while `--mforflatpix' or "
+                "`--mcolforbrightness' have also been given. In such cases "
+                "the profile's pixels will have a value of zero and thus "
+                "they will not be identifiable from the zero-valued "
+                "background. If this behavior is intended, this warning "
+                "can be suppressed with the `--quiet' (or `-q') option.\n");
+          break;
+        }
 }
 
 
@@ -1141,7 +1160,6 @@ ui_prepare_canvas(struct mkprofparams *p)
     }
 
 
-
   /* When individual mode is requested, write the WCS structure to a header
      string to speed up the process: if we don't do it here, this process
      will be necessary on every individual profile's output. So it is much
@@ -1388,8 +1406,10 @@ ui_preparations(struct mkprofparams *p)
   else
     ui_prepare_canvas(p);
 
-  /* Read the (possible) RA/Dec inputs into X and Y for the builder.*/
-  if(p->wcs)
+  /* Read the (possible) RA/Dec inputs into X and Y for the builder.  NOTE:
+     It may happen that there are no input columns, in that case, just
+     ignore this step.*/
+  if(p->wcs && p->num)
     ui_finalize_coordinates(p);
 
   /* Prepare the random number generator. */
@@ -1542,6 +1562,7 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct mkprofparams *p)
   /* Read/allocate all the necessary starting arrays. */
   ui_preparations(p);
 
+
   /* Print introductory information. */
   ui_print_intro(p);
 }
