@@ -5,7 +5,7 @@ NoiseChisel is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -282,8 +282,9 @@ threshold_interp_smooth(struct noisechiselparams *p, gal_data_t **first,
   /* Do the interpolation of both arrays. */
   (*first)->next = *second;
   if(third) (*second)->next = *third;
-  tmp=gal_interpolate_close_neighbors(*first, tl, cp->interpnumngb,
-                                      cp->numthreads, cp->interponlyblank, 1);
+  tmp=gal_interpolate_close_neighbors(*first, tl, cp->interpmetric,
+                                      cp->interpnumngb, cp->numthreads,
+                                      cp->interponlyblank, 1);
   gal_data_free(*first);
   gal_data_free(*second);
   if(third) gal_data_free(*third);
@@ -300,12 +301,13 @@ threshold_interp_smooth(struct noisechiselparams *p, gal_data_t **first,
       (*first)->name="THRESH1_INTERP";
       (*second)->name="THRESH2_INTERP";
       if(third) (*third)->name="THRESH3_INTERP";
-      gal_tile_full_values_write(*first, tl, 1, filename, NULL, PROGRAM_NAME);
-      gal_tile_full_values_write(*second, tl, 1, filename, NULL,
-                                 PROGRAM_NAME);
+      gal_tile_full_values_write(*first, tl, !p->ignoreblankintiles,
+                                 filename, NULL, PROGRAM_NAME);
+      gal_tile_full_values_write(*second, tl, !p->ignoreblankintiles,
+                                 filename, NULL, PROGRAM_NAME);
       if(third)
-        gal_tile_full_values_write(*third, tl, 1, filename, NULL,
-                                   PROGRAM_NAME);
+        gal_tile_full_values_write(*third, tl, !p->ignoreblankintiles,
+                                   filename, NULL, PROGRAM_NAME);
       (*first)->name = (*second)->name = NULL;
       if(third) (*third)->name=NULL;
     }
@@ -340,13 +342,13 @@ threshold_interp_smooth(struct noisechiselparams *p, gal_data_t **first,
           (*first)->name="THRESH1_SMOOTH";
           (*second)->name="THRESH2_SMOOTH";
           if(third) (*third)->name="THRESH3_SMOOTH";
-          gal_tile_full_values_write(*first, tl, 1, filename, NULL,
-                                     PROGRAM_NAME);
-          gal_tile_full_values_write(*second, tl, 1, filename, NULL,
-                                     PROGRAM_NAME);
+          gal_tile_full_values_write(*first, tl, !p->ignoreblankintiles,
+                                     filename, NULL, PROGRAM_NAME);
+          gal_tile_full_values_write(*second, tl, !p->ignoreblankintiles,
+                                     filename, NULL, PROGRAM_NAME);
           if(third)
-            gal_tile_full_values_write(*third, tl, 1, filename, NULL,
-                                       PROGRAM_NAME);
+            gal_tile_full_values_write(*third, tl, !p->ignoreblankintiles,
+                                       filename, NULL, PROGRAM_NAME);
           (*first)->name = (*second)->name = NULL;
           if(third) (*third)->name=NULL;
         }
@@ -518,13 +520,72 @@ qthresh_on_tile(void *in_prm)
 
 
 
+static void
+threshold_good_error(size_t number, int before0_after1, size_t interpnumngb)
+{
+  before0_after1=1;
+
+  /* Set the differing strings. */
+  char *in1 = ( before0_after1
+                ? "after removing outliers"
+                : "for defining a quantile threshold" );
+  char *in2 = ( before0_after1
+                ? ""
+                : "NOTE that this is happening *BEFORE* outlier rejection "
+                  "(where the number may decrease even further).");
+  char *in3 = ( before0_after1
+                ? "\n"
+                  "  - (slightly) Increase `--outliersclip' to reject less "
+                  "as outliers.\n"
+                  "  - (slightly) Increase `--outliersigma' to reject less "
+                  "as outliers.\n"
+                : "\n");
+
+  /* Print the error message and abort. */
+  error(EXIT_FAILURE, 0, "%zu tiles usable %s!\n\n"
+
+        "This is smaller than the requested minimum value of %zu (value to "
+        "the `--interpnumngb' option). %s\n\n"
+
+        "There are several ways to address the problem. The best and most "
+        "highly recommended is to use a larger input if possible (when the "
+        "input is a crop from a larger dataset). If this is not the case, "
+        "or it doesn't solve the problem, you need to loosen the "
+        "parameters mentioned below in the respective order (and therefore "
+        "cause scatter/inaccuracy in the final result). Hence its best to "
+        "not loosen them too much (recall that you can see all the option "
+        "values to Gnuastro's programs by appending `-P' to the end of your "
+        "command).\n"
+        "  - (slightly) Decrease `--tilesize' so your tile-grid has more "
+        "tiles.\n"
+        "  - (slightly) Increase `--meanmedqdiff' to accept more tiles.%s"
+        "  - (slightly) Decrease `--interpnumngb' to be less than %zu.\n\n"
+
+        "---- Tip ----\n"
+        "Append your command with `--checkqthresh' to see the "
+        "successful tiles in relation with this dataset's contents "
+        "before this crash. A visual inspection will greatly help in "
+        "finding the cause/solution for this particular dataset (note "
+        "that the output of `--checkqthresh' is a multi-extension FITS "
+        "file).\n\n"
+        "To better understand this important step, please run the "
+        "following command (press `SPACE'/arrow-keys to navigate and "
+        "`Q' to return back to the command-line):\n\n"
+        "    $ info gnuastro \"Quantifying signal in a tile\"\n", number,
+        in1, interpnumngb, in2, in3, number);
+}
+
+
+
+
+
 void
 threshold_quantile_find_apply(struct noisechiselparams *p)
 {
   char *msg;
-  size_t nval;
   gal_data_t *num;
   struct timeval t1;
+  size_t nval, nblank;
   struct qthreshparams qprm;
   struct gal_options_common_params *cp=&p->cp;
   struct gal_tile_two_layer_params *tl=&cp->tl;
@@ -586,21 +647,28 @@ threshold_quantile_find_apply(struct noisechiselparams *p)
     {
       qprm.erode_th->name="QTHRESH_ERODE";
       qprm.noerode_th->name="QTHRESH_NOERODE";
-      gal_tile_full_values_write(qprm.erode_th, tl, 1, p->qthreshname, NULL,
-                                 PROGRAM_NAME);
-      gal_tile_full_values_write(qprm.noerode_th, tl, 1, p->qthreshname, NULL,
-                                 PROGRAM_NAME);
+      gal_tile_full_values_write(qprm.erode_th, tl,
+                                 !p->ignoreblankintiles,
+                                 p->qthreshname, NULL, PROGRAM_NAME);
+      gal_tile_full_values_write(qprm.noerode_th, tl,
+                                 !p->ignoreblankintiles,
+                                 p->qthreshname, NULL, PROGRAM_NAME);
       qprm.erode_th->name=qprm.noerode_th->name=NULL;
 
       if(qprm.expand_th)
         {
           qprm.expand_th->name="QTHRESH_EXPAND";
-          gal_tile_full_values_write(qprm.expand_th, tl, 1, p->qthreshname,
-                                     NULL, PROGRAM_NAME);
+          gal_tile_full_values_write(qprm.expand_th, tl,
+                                     !p->ignoreblankintiles,
+                                     p->qthreshname, NULL, PROGRAM_NAME);
           qprm.expand_th->name=NULL;
         }
     }
 
+  /* A small sanity check. */
+  nblank=gal_blank_number(qprm.erode_th, 1);
+  if( nblank > qprm.erode_th->size-cp->interpnumngb )
+    threshold_good_error(qprm.erode_th->size-nblank, 0, cp->interpnumngb);
 
   /* Remove outliers if requested. */
   if(p->outliersigma!=0.0)
@@ -616,31 +684,7 @@ threshold_quantile_find_apply(struct noisechiselparams *p)
   num=gal_statistics_number(qprm.erode_th);
   nval=((size_t *)(num->array))[0];
   if( nval < cp->interpnumngb )
-    error(EXIT_FAILURE, 0, "%zu tile(s) can be used for interpolation of the "
-          "quantile threshold values over the full dataset. This is smaller "
-          "than the requested minimum value of %zu (value to the "
-          "`--interpnumngb' option).\n\n"
-          "There are several ways to address the problem. The best and most "
-          "highly recommended is to use a larger input if possible (when the "
-          "input is a crop from a larger dataset). If that is not the case, "
-          "or it doesn't solve the problem, you need to loosen the "
-          "parameters (and therefore cause scatter in the final result). "
-          "Thus don't loosen them too much. Recall that you can see all the "
-          "option values to Gnuastro's programs by appending `-P' to the "
-          "end of your command.\n\n"
-          "  * Slightly decrease `--tilesize' to have more tiles.\n"
-          "  * Slightly increase `--meanmedqdiff' to accept more tiles.\n"
-          "  * Decrease `--outliersigma' to reject less tiles as outliers.\n"
-          "  * Decrease `--interpnumngb' to be smaller than %zu.\n\n"
-          "Append the previous command with `--checkqthresh' to see the "
-          "successful tiles and which were discarded as outliers. This will "
-          "help you find the cause/solution. Note that the output is a "
-          "multi-extension FITS file).\n\n"
-          "To better understand this important step, please run the "
-          "following command (press `SPACE'/arrow-keys to navigate and "
-          "`Q' to return back to the command-line):\n\n"
-          "    $ info gnuastro \"Quantifying signal in a tile\"\n",
-          nval, cp->interpnumngb, cp->interpnumngb);
+    threshold_good_error(nval, 1, cp->interpnumngb);
 
 
   /* Interpolate and smooth the derived values. */
