@@ -5,7 +5,7 @@ NoiseChisel is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -115,6 +115,8 @@ ui_initialize_options(struct noisechiselparams *p,
   cp->numthreads         = gal_threads_number();
   cp->coptions           = gal_commonopts_options;
 
+  /* Program specific initialization. */
+  p->snthresh=NAN;
 
   /* Modify common options. */
   for(i=0; !gal_options_is_last(&cp->coptions[i]); ++i)
@@ -133,6 +135,7 @@ ui_initialize_options(struct noisechiselparams *p,
         case GAL_OPTIONS_KEY_TILESIZE:
         case GAL_OPTIONS_KEY_MINMAPSIZE:
         case GAL_OPTIONS_KEY_NUMCHANNELS:
+        case GAL_OPTIONS_KEY_INTERPMETRIC:
         case GAL_OPTIONS_KEY_INTERPNUMNGB:
         case GAL_OPTIONS_KEY_REMAINDERFRAC:
           cp->coptions[i].mandatory=GAL_OPTIONS_MANDATORY;
@@ -234,9 +237,16 @@ ui_read_check_only_options(struct noisechiselparams *p)
   if(p->openingngb!=4 && p->openingngb!=8)
     error(EXIT_FAILURE, 0, "%zu not acceptable for `--openingngb'. It must "
           "be 4 or 8 (specifying the type of connectivity)", p->openingngb);
+  if(p->dopeningngb!=4 && p->dopeningngb!=8)
+    error(EXIT_FAILURE, 0, "%zu not acceptable for `--dopeningngb'. It must "
+          "be 4 or 8 (specifying the type of connectivity)", p->dopeningngb);
   if(p->holengb!=4 && p->holengb!=8)
     error(EXIT_FAILURE, 0, "%zu not acceptable for `--holengb'. It must "
           "be 4 or 8 (specifying the type of connectivity)", p->holengb);
+  if(p->pseudoconcomp!=4 && p->pseudoconcomp!=8)
+    error(EXIT_FAILURE, 0, "%zu not acceptable for `--pseudoconcomp'. It "
+          "must be 4 or 8 (specifying the type of connectivity)",
+          p->pseudoconcomp);
 
   /* Make sure that the no-erode-quantile is not smaller or equal to
      qthresh. */
