@@ -5,7 +5,7 @@ Statistics is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -468,13 +468,40 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET
     },
     {
-      "modmedqdiff",
-      UI_KEY_MODMEDQDIFF,
+      "meanmedqdiff",
+      UI_KEY_MEANMEDQDIFF,
       "FLT",
       0,
       "Max. mode and median quantile diff. per tile.",
       UI_GROUP_SKY,
-      &p->modmedqdiff,
+      &p->meanmedqdiff,
+      GAL_TYPE_FLOAT32,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "outliersclip",
+      UI_KEY_OUTLIERSCLIP,
+      "FLT,FLT",
+      0,
+      "Sigma-clip params for qthresh outliers.",
+      UI_GROUP_SKY,
+      &p->outliersclip,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      gal_options_read_sigma_clip
+    },
+    {
+      "outliersigma",
+      UI_KEY_OUTLIERSIGMA,
+      "FLT",
+      0,
+      "Multiple of sigma to define outliers.",
+      UI_GROUP_SKY,
+      &p->outliersigma,
       GAL_TYPE_FLOAT32,
       GAL_OPTIONS_RANGE_GE_0,
       GAL_OPTIONS_MANDATORY,
@@ -521,13 +548,13 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET
     },
     {
-      "ignoreblankinsky",
-      UI_KEY_IGNOREBLANKINSKY,
+      "ignoreblankintiles",
+      UI_KEY_IGNOREBLANKINTILES,
       0,
       0,
-      "Don't write input's blanks in the Sky output.",
+      "Don't write input's blanks in the tiled output.",
       UI_GROUP_SKY,
-      &p->ignoreblankinsky,
+      &p->ignoreblankintiles,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
       GAL_OPTIONS_NOT_MANDATORY,
