@@ -5,7 +5,7 @@ Statistics is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -251,6 +251,7 @@ statistics_interpolate_and_write(struct statisticsparams *p,
       && !(p->cp.interponlyblank && gal_blank_present(values, 1)==0) )
     {
       interpd=gal_interpolate_close_neighbors(values, &cp->tl,
+                                              cp->interpmetric,
                                               cp->interpnumngb,
                                               cp->numthreads,
                                               cp->interponlyblank, 0);
@@ -259,7 +260,8 @@ statistics_interpolate_and_write(struct statisticsparams *p,
     }
 
   /* Write the values. */
-  gal_tile_full_values_write(values, &cp->tl, 1, output, NULL, PROGRAM_NAME);
+  gal_tile_full_values_write(values, &cp->tl, !p->ignoreblankintiles,
+                             output, NULL, PROGRAM_NAME);
   gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1);
   gal_fits_key_write_config(&p->cp.okeys, "Statistics configuration",
                             "STATISTICS-CONFIG", output, "0");
