@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2018, Free Software Foundation, Inc.
+Copyright (C) 2017-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -33,6 +33,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/tile.h>
 #include <gnuastro/blank.h>
 #include <gnuastro/pointer.h>
+#include <gnuastro/statistics.h>
 
 #include <gnuastro-internal/checkset.h>
 
@@ -113,313 +114,6 @@ gal_blank_initialize(gal_data_t *input)
 
 
 
-
-/* Return 1 if the contents of the pointer (with the given type) is
-   blank. */
-int
-gal_blank_is(void *pointer, uint8_t type)
-{
-  switch(type)
-    {
-    /* Numeric types */
-    case GAL_TYPE_UINT8:     return *(uint8_t  *)pointer==GAL_BLANK_UINT8;
-    case GAL_TYPE_INT8:      return *(int8_t   *)pointer==GAL_BLANK_INT8;
-    case GAL_TYPE_UINT16:    return *(uint16_t *)pointer==GAL_BLANK_UINT16;
-    case GAL_TYPE_INT16:     return *(int16_t  *)pointer==GAL_BLANK_INT16;
-    case GAL_TYPE_UINT32:    return *(uint32_t *)pointer==GAL_BLANK_UINT32;
-    case GAL_TYPE_INT32:     return *(int32_t  *)pointer==GAL_BLANK_INT32;
-    case GAL_TYPE_UINT64:    return *(uint64_t *)pointer==GAL_BLANK_UINT64;
-    case GAL_TYPE_INT64:     return *(int64_t  *)pointer==GAL_BLANK_INT64;
-    case GAL_TYPE_FLOAT32:   return isnan( *(float *)(pointer) );
-    case GAL_TYPE_FLOAT64:   return isnan( *(double *)(pointer) );
-
-    /* String. */
-    case GAL_TYPE_STRING:    if(!strcmp(pointer,GAL_BLANK_STRING)) return 1;
-
-    /* Complex types */
-    case GAL_TYPE_COMPLEX32:
-    case GAL_TYPE_COMPLEX64:
-      error(EXIT_FAILURE, 0, "%s: complex types are not yet supported",
-            __func__);
-
-    /* Bit. */
-    case GAL_TYPE_BIT:
-      error(EXIT_FAILURE, 0, "%s: bit type datasets are not yet supported",
-            __func__);
-
-    default:
-      error(EXIT_FAILURE, 0, "%s: type value (%d) not recognized",
-            __func__, type);
-    }
-
-  /* Control should not reach here, so print an error if it does, then
-     return a 0 (just to avoid compiler warnings). */
-  error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to address the "
-        "problem. Control should not reach the end of this function",
-        __func__, PACKAGE_BUGREPORT);
-  return 0;
-}
-
-
-
-
-
-/* Return 1 if the dataset has a blank value and zero if it doesn't. Before
-   checking the dataset, this function will look at its flags. If the
-   `GAL_DATA_FLAG_HASBLANK' or `GAL_DATA_FLAG_DONT_CHECK_ZERO' bits of
-   `input->flag' are set to 1, this function will not do any check and will
-   just use the information in the flags.
-
-   If you want to re-check a dataset which has non-zero flags, then
-   explicitly set the appropriate flag to zero before calling this
-   function. When there are no other flags, you can just set `input->flags'
-   to zero, otherwise you can use this expression:
-
-       input->flags &= ~ (GAL_DATA_FLAG_HASBLANK | GAL_DATA_FLAG_USE_ZERO);
-
-   This function has no side-effects on the dataset: it will not toggle the
-   flags, to avoid repeating parsing of the full dataset multiple times
-   (when it occurs), please toggle the flags your self after the first
-   check. */
-#define HAS_BLANK(IT) {                                                 \
-    IT b, *a=input->array, *af=a+input->size, *start;                   \
-    gal_blank_write(&b, block->type);                                   \
-                                                                        \
-    /* If this is a tile, not a full block. */                          \
-    if(input!=block)                                                    \
-      start=gal_tile_start_end_ind_inclusive(input, block, start_end_inc); \
-                                                                        \
-    /* Go over all the elements. */                                     \
-    while( start_end_inc[0] + increment <= start_end_inc[1] )           \
-      {                                                                 \
-        /* Necessary when we are on a tile. */                          \
-        if(input!=block)                                                \
-          af = ( a = start + increment ) + input->dsize[input->ndim-1]; \
-                                                                        \
-        /* Check for blank values. */                                   \
-        if(b==b) do if(*a==b)  { hasblank=1; break; } while(++a<af);    \
-        else     do if(*a!=*a) { hasblank=1; break; } while(++a<af);    \
-                                                                        \
-        /* Necessary when we are on a tile. */                          \
-        if(input!=block)                                                \
-          increment += gal_tile_block_increment(block, input->dsize,    \
-                                                num_increment++, NULL); \
-        else break;                                                     \
-      }                                                                 \
-  }
-int
-gal_blank_present(gal_data_t *input, int updateflag)
-{
-  int hasblank=0;
-  char **str, **strf;
-  size_t increment=0, num_increment=1;
-  gal_data_t *block=gal_tile_block(input);
-  size_t start_end_inc[2]={0,block->size-1}; /* -1: this is INCLUSIVE. */
-
-  /* If there is nothing in the array (its size is zero), then return 0 (no
-     blank is present. */
-  if(input->size==0) return 0;
-
-  /* From the user's flags, you can tell if the dataset has already been
-     checked for blank values or not. If it has, then just return the
-     checked result. */
-  if( input->flag & GAL_DATA_FLAG_BLANK_CH )
-    return input->flag & GAL_DATA_FLAG_HASBLANK;
-
-  /* Go over the pixels and check: */
-  switch(block->type)
-    {
-    /* Numeric types */
-    case GAL_TYPE_UINT8:     HAS_BLANK( uint8_t  );    break;
-    case GAL_TYPE_INT8:      HAS_BLANK( int8_t   );    break;
-    case GAL_TYPE_UINT16:    HAS_BLANK( uint16_t );    break;
-    case GAL_TYPE_INT16:     HAS_BLANK( int16_t  );    break;
-    case GAL_TYPE_UINT32:    HAS_BLANK( uint32_t );    break;
-    case GAL_TYPE_INT32:     HAS_BLANK( int32_t  );    break;
-    case GAL_TYPE_UINT64:    HAS_BLANK( uint64_t );    break;
-    case GAL_TYPE_INT64:     HAS_BLANK( int64_t  );    break;
-    case GAL_TYPE_FLOAT32:   HAS_BLANK( float    );    break;
-    case GAL_TYPE_FLOAT64:   HAS_BLANK( double   );    break;
-
-    /* String. */
-    case GAL_TYPE_STRING:
-      if(input!=block)
-        error(EXIT_FAILURE, 0, "%s: tile mode is currently not supported for "
-              "strings", __func__);
-      strf = (str=input->array) + input->size;
-      do if(!strcmp(*str,GAL_BLANK_STRING)) return 1; while(++str<strf);
-      break;
-
-    /* Complex types */
-    case GAL_TYPE_COMPLEX32:
-    case GAL_TYPE_COMPLEX64:
-      error(EXIT_FAILURE, 0, "%s: complex types are not yet supported",
-            __func__);
-
-    /* Bit */
-    case GAL_TYPE_BIT:
-      error(EXIT_FAILURE, 0, "%s: bit type datasets are not yet supported",
-            __func__);
-
-    default:
-      error(EXIT_FAILURE, 0, "%s: type value (%d) not recognized",
-            __func__, block->type);
-    }
-
-  /* Update the flag if requested. */
-  if(updateflag)
-    {
-      input->flag |= GAL_DATA_FLAG_BLANK_CH;
-      if(hasblank) input->flag |= GAL_DATA_FLAG_HASBLANK;
-      else         input->flag &= ~GAL_DATA_FLAG_HASBLANK;
-    }
-
-  /* If there was a blank value, then the function would have returned with
-     a value of 1. So if it reaches here, then we can be sure that there
-     was no blank values, hence, return 0. */
-  return hasblank;
-}
-
-
-
-
-
-
-
-/* Create a dataset of the the same size as the input, but with an uint8_t
-   type that has a value of 1 for data that are blank and 0 for those that
-   aren't. */
-#define FLAG_BLANK(IT) {                                                \
-    IT b, *a=input->array;                                              \
-    gal_blank_write(&b, input->type);                                   \
-    if(b==b) /* Blank value can be checked with the equal comparison */ \
-      do { *o = *a==b;  ++a; } while(++o<of);                           \
-    else     /* Blank value will fail with the equal comparison */      \
-      do { *o = *a!=*a; ++a; } while(++o<of);                           \
-  }
-gal_data_t *
-gal_blank_flag(gal_data_t *input)
-{
-  uint8_t *o, *of;
-  gal_data_t *out;
-  char **str=input->array, **strf=str+input->size;
-
-  if( gal_blank_present(input, 0) )
-    {
-      /* Allocate a non-cleared output array, we are going to parse the
-         input and fill in each element. */
-      out=gal_data_alloc(NULL, GAL_TYPE_UINT8, input->ndim, input->dsize,
-                         input->wcs, 0, input->minmapsize, NULL, "bool",
-                         NULL);
-
-      /* Set the pointers for easy looping. */
-      of=(o=out->array)+input->size;
-
-      /* Go over the pixels and set the output values. */
-      switch(input->type)
-        {
-          /* Numeric types */
-        case GAL_TYPE_UINT8:     FLAG_BLANK( uint8_t  );    break;
-        case GAL_TYPE_INT8:      FLAG_BLANK( int8_t   );    break;
-        case GAL_TYPE_UINT16:    FLAG_BLANK( uint16_t );    break;
-        case GAL_TYPE_INT16:     FLAG_BLANK( int16_t  );    break;
-        case GAL_TYPE_UINT32:    FLAG_BLANK( uint32_t );    break;
-        case GAL_TYPE_INT32:     FLAG_BLANK( int32_t  );    break;
-        case GAL_TYPE_UINT64:    FLAG_BLANK( uint64_t );    break;
-        case GAL_TYPE_INT64:     FLAG_BLANK( int64_t  );    break;
-        case GAL_TYPE_FLOAT32:   FLAG_BLANK( float    );    break;
-        case GAL_TYPE_FLOAT64:   FLAG_BLANK( double   );    break;
-
-          /* String. */
-        case GAL_TYPE_STRING:
-          do *o++ = !strcmp(*str,GAL_BLANK_STRING); while(++str<strf);
-          break;
-
-          /* Currently unsupported types. */
-        case GAL_TYPE_BIT:
-        case GAL_TYPE_COMPLEX32:
-        case GAL_TYPE_COMPLEX64:
-          error(EXIT_FAILURE, 0, "%s: %s type not yet supported",
-                __func__, gal_type_name(input->type, 1));
-
-          /* Bad input. */
-        default:
-          error(EXIT_FAILURE, 0, "%s: type value (%d) not recognized",
-                __func__, input->type);
-        }
-    }
-  else
-    /* Allocate a CLEAR data structure (all zeros). */
-    out=gal_data_alloc(NULL, GAL_TYPE_UINT8, input->ndim, input->dsize,
-                       input->wcs, 1, input->minmapsize, NULL, "bool", NULL);
-
-  /* Return */
-  return out;
-}
-
-
-
-
-
-/* Remove blank elements from a dataset, convert it to a 1D dataset and
-   adjust the size properly. In practice this function doesn't `realloc'
-   the input array, all it does is to shift the blank eleemnts to the end
-   and adjust the size elements of the `gal_data_t'. */
-#define BLANK_REMOVE(IT) {                                              \
-    IT b, *a=input->array, *af=a+input->size, *o=input->array;          \
-    gal_blank_write(&b, input->type);                                   \
-    if(b==b)       /* Blank value can be be checked with equal. */      \
-      do if(*a!=b)  { ++num; *o++=*a; } while(++a<af);                  \
-    else           /* Blank value will fail on equal comparison. */     \
-      do if(*a==*a) { ++num; *o++=*a; } while(++a<af);                  \
-  }
-void
-gal_blank_remove(gal_data_t *input)
-{
-  size_t num=0;
-
-  /* This function currently assumes a contiguous patch of memory. */
-  if(input->block && input->ndim!=1 )
-    error(EXIT_FAILURE, 0, "%s: tiles in datasets with more dimensions than "
-          "1 are not yet supported. Your input has %zu dimensions",
-          __func__, input->ndim);
-
-  /* If the dataset doesn't have blank values, then just get the size. */
-  if( gal_blank_present(input, 0) )
-    {
-      /* Shift all non-blank elements to the start of the array. */
-      switch(input->type)
-        {
-        case GAL_TYPE_UINT8:    BLANK_REMOVE( uint8_t  );    break;
-        case GAL_TYPE_INT8:     BLANK_REMOVE( int8_t   );    break;
-        case GAL_TYPE_UINT16:   BLANK_REMOVE( uint16_t );    break;
-        case GAL_TYPE_INT16:    BLANK_REMOVE( int16_t  );    break;
-        case GAL_TYPE_UINT32:   BLANK_REMOVE( uint32_t );    break;
-        case GAL_TYPE_INT32:    BLANK_REMOVE( int32_t  );    break;
-        case GAL_TYPE_UINT64:   BLANK_REMOVE( uint64_t );    break;
-        case GAL_TYPE_INT64:    BLANK_REMOVE( int64_t  );    break;
-        case GAL_TYPE_FLOAT32:  BLANK_REMOVE( float    );    break;
-        case GAL_TYPE_FLOAT64:  BLANK_REMOVE( double   );    break;
-        default:
-          error(EXIT_FAILURE, 0, "%s: type code %d not recognized",
-                __func__, input->type);
-        }
-    }
-  else num=input->size;
-
-  /* Adjust the size elements of the dataset. */
-  input->ndim=1;
-  input->dsize[0]=input->size=num;
-
-  /* Set the flags to mark that there is no blanks. */
-  input->flag |= GAL_DATA_FLAG_BLANK_CH;
-  input->flag &= ~GAL_DATA_FLAG_HASBLANK;
-}
-
-
-
-
-
 /* Print the blank value as a string. For the integer types, we'll use the
    PRIxNN keywords of `inttypes.h' (which is imported into Gnuastro from
    Gnulib, so we don't necessarily rely on the host system having it). */
@@ -593,3 +287,409 @@ gal_blank_as_string(uint8_t type, int width)
     }
   return blank;
 }
+
+
+
+
+
+
+/* Return 1 if the contents of the pointer (with the given type) is
+   blank. */
+int
+gal_blank_is(void *pointer, uint8_t type)
+{
+  switch(type)
+    {
+    /* Numeric types */
+    case GAL_TYPE_UINT8:     return *(uint8_t  *)pointer==GAL_BLANK_UINT8;
+    case GAL_TYPE_INT8:      return *(int8_t   *)pointer==GAL_BLANK_INT8;
+    case GAL_TYPE_UINT16:    return *(uint16_t *)pointer==GAL_BLANK_UINT16;
+    case GAL_TYPE_INT16:     return *(int16_t  *)pointer==GAL_BLANK_INT16;
+    case GAL_TYPE_UINT32:    return *(uint32_t *)pointer==GAL_BLANK_UINT32;
+    case GAL_TYPE_INT32:     return *(int32_t  *)pointer==GAL_BLANK_INT32;
+    case GAL_TYPE_UINT64:    return *(uint64_t *)pointer==GAL_BLANK_UINT64;
+    case GAL_TYPE_INT64:     return *(int64_t  *)pointer==GAL_BLANK_INT64;
+    case GAL_TYPE_FLOAT32:   return isnan( *(float *)(pointer) );
+    case GAL_TYPE_FLOAT64:   return isnan( *(double *)(pointer) );
+
+    /* String. */
+    case GAL_TYPE_STRING:    if(!strcmp(pointer,GAL_BLANK_STRING)) return 1;
+
+    /* Complex types */
+    case GAL_TYPE_COMPLEX32:
+    case GAL_TYPE_COMPLEX64:
+      error(EXIT_FAILURE, 0, "%s: complex types are not yet supported",
+            __func__);
+
+    /* Bit. */
+    case GAL_TYPE_BIT:
+      error(EXIT_FAILURE, 0, "%s: bit type datasets are not yet supported",
+            __func__);
+
+    default:
+      error(EXIT_FAILURE, 0, "%s: type value (%d) not recognized",
+            __func__, type);
+    }
+
+  /* Control should not reach here, so print an error if it does, then
+     return a 0 (just to avoid compiler warnings). */
+  error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to address the "
+        "problem. Control should not reach the end of this function",
+        __func__, PACKAGE_BUGREPORT);
+  return 0;
+}
+
+
+
+
+
+/* Return 1 if the dataset has a blank value and zero if it doesn't. Before
+   checking the dataset, this function will look at its flags. If the
+   `GAL_DATA_FLAG_HASBLANK' or `GAL_DATA_FLAG_DONT_CHECK_ZERO' bits of
+   `input->flag' are set to 1, this function will not do any check and will
+   just use the information in the flags.
+
+   If you want to re-check a dataset which has non-zero flags, then
+   explicitly set the appropriate flag to zero before calling this
+   function. When there are no other flags, you can just set `input->flags'
+   to zero, otherwise you can use this expression:
+
+       input->flags &= ~ (GAL_DATA_FLAG_HASBLANK | GAL_DATA_FLAG_USE_ZERO);
+
+   This function has no side-effects on the dataset: it will not toggle the
+   flags, to avoid repeating parsing of the full dataset multiple times
+   (when it occurs), please toggle the flags your self after the first
+   check. */
+#define HAS_BLANK(IT) {                                                 \
+    IT b, *a=input->array, *af=a+input->size, *start;                   \
+    gal_blank_write(&b, block->type);                                   \
+                                                                        \
+    /* If this is a tile, not a full block. */                          \
+    if(input!=block)                                                    \
+      start=gal_tile_start_end_ind_inclusive(input, block, start_end_inc); \
+                                                                        \
+    /* Go over all the elements. */                                     \
+    while( start_end_inc[0] + increment <= start_end_inc[1] )           \
+      {                                                                 \
+        /* Necessary when we are on a tile. */                          \
+        if(input!=block)                                                \
+          af = ( a = start + increment ) + input->dsize[input->ndim-1]; \
+                                                                        \
+        /* Check for blank values. */                                   \
+        if(b==b) do if(*a==b)  { hasblank=1; break; } while(++a<af);    \
+        else     do if(*a!=*a) { hasblank=1; break; } while(++a<af);    \
+                                                                        \
+        /* Necessary when we are on a tile. */                          \
+        if(input!=block)                                                \
+          increment += gal_tile_block_increment(block, input->dsize,    \
+                                                num_increment++, NULL); \
+        else break;                                                     \
+      }                                                                 \
+  }
+int
+gal_blank_present(gal_data_t *input, int updateflag)
+{
+  int hasblank=0;
+  char **str, **strf;
+  size_t increment=0, num_increment=1;
+  gal_data_t *block=gal_tile_block(input);
+  size_t start_end_inc[2]={0,block->size-1}; /* -1: this is INCLUSIVE. */
+
+  /* If there is nothing in the array (its size is zero), then return 0 (no
+     blank is present. */
+  if(input->size==0) return 0;
+
+  /* From the user's flags, you can tell if the dataset has already been
+     checked for blank values or not. If it has, then just return the
+     checked result. */
+  if( input->flag & GAL_DATA_FLAG_BLANK_CH )
+    return input->flag & GAL_DATA_FLAG_HASBLANK;
+
+  /* Go over the pixels and check: */
+  switch(block->type)
+    {
+    /* Numeric types */
+    case GAL_TYPE_UINT8:     HAS_BLANK( uint8_t  );    break;
+    case GAL_TYPE_INT8:      HAS_BLANK( int8_t   );    break;
+    case GAL_TYPE_UINT16:    HAS_BLANK( uint16_t );    break;
+    case GAL_TYPE_INT16:     HAS_BLANK( int16_t  );    break;
+    case GAL_TYPE_UINT32:    HAS_BLANK( uint32_t );    break;
+    case GAL_TYPE_INT32:     HAS_BLANK( int32_t  );    break;
+    case GAL_TYPE_UINT64:    HAS_BLANK( uint64_t );    break;
+    case GAL_TYPE_INT64:     HAS_BLANK( int64_t  );    break;
+    case GAL_TYPE_FLOAT32:   HAS_BLANK( float    );    break;
+    case GAL_TYPE_FLOAT64:   HAS_BLANK( double   );    break;
+
+    /* String. */
+    case GAL_TYPE_STRING:
+      if(input!=block)
+        error(EXIT_FAILURE, 0, "%s: tile mode is currently not supported for "
+              "strings", __func__);
+      strf = (str=input->array) + input->size;
+      do if(!strcmp(*str,GAL_BLANK_STRING)) return 1; while(++str<strf);
+      break;
+
+    /* Complex types */
+    case GAL_TYPE_COMPLEX32:
+    case GAL_TYPE_COMPLEX64:
+      error(EXIT_FAILURE, 0, "%s: complex types are not yet supported",
+            __func__);
+
+    /* Bit */
+    case GAL_TYPE_BIT:
+      error(EXIT_FAILURE, 0, "%s: bit type datasets are not yet supported",
+            __func__);
+
+    default:
+      error(EXIT_FAILURE, 0, "%s: type value (%d) not recognized",
+            __func__, block->type);
+    }
+
+  /* Update the flag if requested. */
+  if(updateflag)
+    {
+      input->flag |= GAL_DATA_FLAG_BLANK_CH;
+      if(hasblank) input->flag |= GAL_DATA_FLAG_HASBLANK;
+      else         input->flag &= ~GAL_DATA_FLAG_HASBLANK;
+    }
+
+  /* If there was a blank value, then the function would have returned with
+     a value of 1. So if it reaches here, then we can be sure that there
+     was no blank values, hence, return 0. */
+  return hasblank;
+}
+
+
+
+
+
+/* Return the number of blank elements in the dataset. */
+size_t
+gal_blank_number(gal_data_t *input, int updateflag)
+{
+  gal_data_t *number;
+  size_t num_not_blank;
+
+  if(input)
+    {
+      if( gal_blank_present(input, updateflag) )
+        {
+          number=gal_statistics_number(input);
+          num_not_blank=((size_t *)(number->array))[0];
+          gal_data_free(number);
+          return input->size - num_not_blank;
+        }
+      else
+        return 0;
+    }
+  else
+    return GAL_BLANK_SIZE_T;
+}
+
+
+
+
+
+
+
+/* Create a dataset of the the same size as the input, but with an uint8_t
+   type that has a value of 1 for data that are blank and 0 for those that
+   aren't. */
+#define FLAG_BLANK(IT) {                                                \
+    IT b, *a=input->array;                                              \
+    gal_blank_write(&b, input->type);                                   \
+    if(b==b) /* Blank value can be checked with the equal comparison */ \
+      do { *o = *a==b;  ++a; } while(++o<of);                           \
+    else     /* Blank value will fail with the equal comparison */      \
+      do { *o = *a!=*a; ++a; } while(++o<of);                           \
+  }
+gal_data_t *
+gal_blank_flag(gal_data_t *input)
+{
+  uint8_t *o, *of;
+  gal_data_t *out;
+  char **str=input->array, **strf=str+input->size;
+
+  if( gal_blank_present(input, 0) )
+    {
+      /* Allocate a non-cleared output array, we are going to parse the
+         input and fill in each element. */
+      out=gal_data_alloc(NULL, GAL_TYPE_UINT8, input->ndim, input->dsize,
+                         input->wcs, 0, input->minmapsize, NULL, "bool",
+                         NULL);
+
+      /* Set the pointers for easy looping. */
+      of=(o=out->array)+input->size;
+
+      /* Go over the pixels and set the output values. */
+      switch(input->type)
+        {
+        /* Numeric types */
+        case GAL_TYPE_UINT8:     FLAG_BLANK( uint8_t  );    break;
+        case GAL_TYPE_INT8:      FLAG_BLANK( int8_t   );    break;
+        case GAL_TYPE_UINT16:    FLAG_BLANK( uint16_t );    break;
+        case GAL_TYPE_INT16:     FLAG_BLANK( int16_t  );    break;
+        case GAL_TYPE_UINT32:    FLAG_BLANK( uint32_t );    break;
+        case GAL_TYPE_INT32:     FLAG_BLANK( int32_t  );    break;
+        case GAL_TYPE_UINT64:    FLAG_BLANK( uint64_t );    break;
+        case GAL_TYPE_INT64:     FLAG_BLANK( int64_t  );    break;
+        case GAL_TYPE_FLOAT32:   FLAG_BLANK( float    );    break;
+        case GAL_TYPE_FLOAT64:   FLAG_BLANK( double   );    break;
+
+        /* String. */
+        case GAL_TYPE_STRING:
+          do *o++ = !strcmp(*str,GAL_BLANK_STRING); while(++str<strf);
+          break;
+
+        /* Currently unsupported types. */
+        case GAL_TYPE_BIT:
+        case GAL_TYPE_COMPLEX32:
+        case GAL_TYPE_COMPLEX64:
+          error(EXIT_FAILURE, 0, "%s: %s type not yet supported",
+                __func__, gal_type_name(input->type, 1));
+
+        /* Bad input. */
+        default:
+          error(EXIT_FAILURE, 0, "%s: type value (%d) not recognized",
+                __func__, input->type);
+        }
+    }
+  else
+    /* Allocate a CLEAR data structure (all zeros). */
+    out=gal_data_alloc(NULL, GAL_TYPE_UINT8, input->ndim, input->dsize,
+                       input->wcs, 1, input->minmapsize, NULL, "bool", NULL);
+
+  /* Return */
+  return out;
+}
+
+
+
+
+
+/* Write a blank value in the input anywhere that the flag dataset is not
+   zero or not blank. */
+#define BLANK_FLAG_APPLY(IT) {                                          \
+    IT *i=input->array, b;                                              \
+    gal_blank_write(&b, input->type);                                   \
+    do {if(*f && *f!=GAL_BLANK_UINT8) *i=b; ++i;} while(++f<ff);        \
+  }
+void
+gal_blank_flag_apply(gal_data_t *input, gal_data_t *flag)
+{
+  char **str=input->array;
+  uint8_t *f=flag->array, *ff=f+flag->size;
+
+  /* Sanity check. */
+  if(flag->type!=GAL_TYPE_UINT8)
+    error(EXIT_FAILURE, 0, "%s: the `flag' argument has a `%s' type, it "
+          "must have an unsigned 8-bit type", __func__,
+          gal_type_name(flag->type, 1));
+  if(gal_dimension_is_different(input, flag))
+    error(EXIT_FAILURE, 0, "%s: the `flag' argument doesn't have the same "
+          "size as the `input' argument", __func__);
+
+  /* Write the blank values. */
+  switch(input->type)
+    {
+    /* Basic numeric types. */
+    case GAL_TYPE_UINT8:     BLANK_FLAG_APPLY( uint8_t  );    break;
+    case GAL_TYPE_INT8:      BLANK_FLAG_APPLY( int8_t   );    break;
+    case GAL_TYPE_UINT16:    BLANK_FLAG_APPLY( uint16_t );    break;
+    case GAL_TYPE_INT16:     BLANK_FLAG_APPLY( int16_t  );    break;
+    case GAL_TYPE_UINT32:    BLANK_FLAG_APPLY( uint32_t );    break;
+    case GAL_TYPE_INT32:     BLANK_FLAG_APPLY( int32_t  );    break;
+    case GAL_TYPE_UINT64:    BLANK_FLAG_APPLY( uint64_t );    break;
+    case GAL_TYPE_INT64:     BLANK_FLAG_APPLY( int64_t  );    break;
+    case GAL_TYPE_FLOAT32:   BLANK_FLAG_APPLY( float    );    break;
+    case GAL_TYPE_FLOAT64:   BLANK_FLAG_APPLY( double   );    break;
+
+    /* Strings. */
+    case GAL_TYPE_STRING:
+      do
+        {
+          if(*f && *f!=GAL_BLANK_UINT8)
+            {
+              free(*str);
+              *str=gal_blank_as_string(GAL_TYPE_STRING, 0);
+            }
+          ++str;
+        }
+      while(++f<ff);
+      break;
+
+    /* Currently unsupported types. */
+    case GAL_TYPE_BIT:
+    case GAL_TYPE_COMPLEX32:
+    case GAL_TYPE_COMPLEX64:
+      error(EXIT_FAILURE, 0, "%s: %s type not yet supported",
+            __func__, gal_type_name(input->type, 1));
+
+    /* Bad input. */
+    default:
+      error(EXIT_FAILURE, 0, "%s: type value (%d) not recognized",
+            __func__, input->type);
+    }
+
+  /* Update the blank flags. */
+  gal_blank_present(input, 1);
+}
+
+
+
+
+
+/* Remove blank elements from a dataset, convert it to a 1D dataset and
+   adjust the size properly. In practice this function doesn't `realloc'
+   the input array, all it does is to shift the blank eleemnts to the end
+   and adjust the size elements of the `gal_data_t'. */
+#define BLANK_REMOVE(IT) {                                              \
+    IT b, *a=input->array, *af=a+input->size, *o=input->array;          \
+    gal_blank_write(&b, input->type);                                   \
+    if(b==b)       /* Blank value can be be checked with equal. */      \
+      do if(*a!=b)  { ++num; *o++=*a; } while(++a<af);                  \
+    else           /* Blank value will fail on equal comparison. */     \
+      do if(*a==*a) { ++num; *o++=*a; } while(++a<af);                  \
+  }
+void
+gal_blank_remove(gal_data_t *input)
+{
+  size_t num=0;
+
+  /* This function currently assumes a contiguous patch of memory. */
+  if(input->block && input->ndim!=1 )
+    error(EXIT_FAILURE, 0, "%s: tiles in datasets with more dimensions than "
+          "1 are not yet supported. Your input has %zu dimensions",
+          __func__, input->ndim);
+
+  /* If the dataset doesn't have blank values, then just get the size. */
+  if( gal_blank_present(input, 0) )
+    {
+      /* Shift all non-blank elements to the start of the array. */
+      switch(input->type)
+        {
+        case GAL_TYPE_UINT8:    BLANK_REMOVE( uint8_t  );    break;
+        case GAL_TYPE_INT8:     BLANK_REMOVE( int8_t   );    break;
+        case GAL_TYPE_UINT16:   BLANK_REMOVE( uint16_t );    break;
+        case GAL_TYPE_INT16:    BLANK_REMOVE( int16_t  );    break;
+        case GAL_TYPE_UINT32:   BLANK_REMOVE( uint32_t );    break;
+        case GAL_TYPE_INT32:    BLANK_REMOVE( int32_t  );    break;
+        case GAL_TYPE_UINT64:   BLANK_REMOVE( uint64_t );    break;
+        case GAL_TYPE_INT64:    BLANK_REMOVE( int64_t  );    break;
+        case GAL_TYPE_FLOAT32:  BLANK_REMOVE( float    );    break;
+        case GAL_TYPE_FLOAT64:  BLANK_REMOVE( double   );    break;
+        default:
+          error(EXIT_FAILURE, 0, "%s: type code %d not recognized",
+                __func__, input->type);
+        }
+    }
+  else num=input->size;
+
+  /* Adjust the size elements of the dataset. */
+  input->ndim=1;
+  input->dsize[0]=input->size=num;
+
+  /* Set the flags to mark that there is no blanks. */
+  input->flag |= GAL_DATA_FLAG_BLANK_CH;
+  input->flag &= ~GAL_DATA_FLAG_HASBLANK;
+}
