@@ -3,9 +3,9 @@ Fixed strings for use in all utilities.
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -26,7 +26,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 
 #define GAL_STRINGS_SHORT_COPYRIGHT                                     \
-  "Copyright (C) 2015-2017, Free Software Foundation, Inc."
+  "Copyright (C) 2015-2019, Free Software Foundation, Inc."
 
 
 #define GAL_STRINGS_SHORT_LICENSE                                       \
