@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -1186,7 +1186,7 @@ enum is_sorted_return
 int
 gal_statistics_is_sorted(gal_data_t *input, int updateflags)
 {
-  int out;
+  int out=GAL_BLANK_INT16; /* On some systems, int may be 16-bits wide. */
 
   /* If the flags are already set, don't bother going over the dataset. */
   if( input->flag & GAL_DATA_FLAG_SORT_CH )
@@ -1206,8 +1206,9 @@ gal_statistics_is_sorted(gal_data_t *input, int updateflags)
        increasing. */
     case 1:
       out=STATISTICS_IS_SORTED_INCREASING;
+      break;
 
-    /* Do the check. */
+    /* Do the check when there is more than one element. */
     default:
       switch(input->type)
         {
@@ -1397,7 +1398,6 @@ gal_statistics_no_blank_sorted(gal_data_t *input, int inplace)
         }
       else noblank=contig;
 
-
       /* If there are any non-blank elements, make sure the array is
          sorted. After this step, we won't be dealing with `noblank' any
          more but with `sorted'. */
@@ -1421,9 +1421,14 @@ gal_statistics_no_blank_sorted(gal_data_t *input, int inplace)
       else
         sorted=noblank;
     }
-  /* When the input's size is zero, just return the actual input. */
+
+  /* Input's size was zero. Note that we cannot simply copy the zero-sized
+     input dataset, we'll have to allocate it here. */
   else
-    sorted = inplace ? input : gal_data_copy(input);
+    sorted = ( inplace
+               ? input
+               : gal_data_alloc(NULL, input->type, 0, NULL, input->wcs, 0,
+                                input->minmapsize, NULL, NULL, NULL) );
 
   /* Set the blank and sorted flags if the dataset has zero-elements. Even
      if having blank values or being sorted is not defined on a
@@ -1439,7 +1444,6 @@ gal_statistics_no_blank_sorted(gal_data_t *input, int inplace)
       sorted->flag &= ~GAL_DATA_FLAG_SORTED_D;
     }
 
-
   /* Return final array. */
   return sorted;
 }
@@ -2008,20 +2012,29 @@ gal_statistics_sigma_clip(gal_data_t *input, float multip, float param,
       start=nbs->array;
       while(num<maxnum && size)
         {
-          /* Find the median. */
-          statistics_median_in_sorted_no_blank(nbs, median_i->array);
-          median_d=gal_data_copy_to_new_type(median_i, GAL_TYPE_FLOAT64);
-
           /* Find the average and Standard deviation, note that both
              `start' and `size' will be different in the next round. */
           nbs->array = start;
           nbs->size = oldsize = size;
-          meanstd=gal_statistics_mean_std(nbs);
 
-          /* Put the three final values in usable (with a type)
-             pointers. */
-          med  = median_d->array;
+          /* For a detailed check, just correct the type).
+          if(!quiet)
+            {
+              size_t iii;
+              printf("nbs->size: %zu\n", nbs->size);
+              for(iii=0;iii<nbs->size;++iii)
+                printf("%f\n", ((float *)(nbs->array))[iii]);
+            }
+          */
+
+          /* Find the mean, median and standard deviation. */
+          meanstd=gal_statistics_mean_std(nbs);
+          statistics_median_in_sorted_no_blank(nbs, median_i->array);
+          median_d=gal_data_copy_to_new_type(median_i, GAL_TYPE_FLOAT64);
+
+          /* Put them in usable (with a type) pointers. */
           mean = meanstd->array;
+          med  = median_d->array;
           std  = &((double *)(meanstd->array))[1];
 
           /* If the user wanted to view the steps, show it to them. */
@@ -2145,6 +2158,9 @@ gal_statistics_outlier_positive(gal_data_t *input, size_t window_size,
   /* Remove all blanks and sort the dataset. */
   nbs=gal_statistics_no_blank_sorted(input, inplace);
 
+  /* If all elements are blank, simply return the default (NULL) output. */
+  if(nbs->size==0) return out;
+
   /* Only continue if the window size is more than 2 elements (out
      "outlier" is hard to define on smaller datasets). */
   if(window_size>2)
@@ -2192,3 +2208,92 @@ gal_statistics_outlier_positive(gal_data_t *input, size_t window_size,
   if(nbs!=input) gal_data_free(nbs);
   return out;
 }
+
+
+
+
+
+
+/* Find the outliers using the first flat portion of the cumulative
+   frequency plot. A good value for window and thresh are 5 and 1.0. */
+#define OUTLIER_FLAT_CFP_BYTYPE(IT) {                                   \
+    IT m, n, *a=nbs->array, *p=a+dist, *pp=a+nbs->size-dist;            \
+    min=a[0]; max=a[nbs->size-1];                                       \
+    do                                                                  \
+      {                                                                 \
+        m=( *(p-dist)-min ) / (max-min) * 100.0f;                       \
+        n=( *(p+dist)-min ) / (max-min) * 100.0f;                       \
+        check=1/(n-m);                                                  \
+        if(!quiet) printf("%-6zu%-15g%-15g\n", p-a, (float)(*p), check); \
+        if( check<thresh && start==GAL_BLANK_SIZE_T )                   \
+          { start=p-a; if(!quiet) printf("\t---start\n"); }             \
+        if( check>thresh && start!=GAL_BLANK_SIZE_T )                   \
+          {                                                             \
+             widthcheck=100.0 * (double)(*p-a[start])/(max-min);        \
+             if(!quiet) printf("\t----end: %f\n", widthcheck);          \
+             if( widthcheck > width ) { flatind=start; break; }         \
+             start=GAL_BLANK_SIZE_T;                                    \
+          }                                                             \
+      }                                                                 \
+    while(++p<pp);                                                      \
+    if( pp==p && start!=GAL_BLANK_SIZE_T )                              \
+      {                                                                 \
+        widthcheck=100.0 * (double)(*(p-1)-a[start])/(max-min);         \
+        if(!quiet) printf("\t----end: %f\n", widthcheck);               \
+        if( widthcheck > width ) flatind=start;                         \
+      }                                                                 \
+  }
+
+gal_data_t *
+gal_statistics_outlier_flat_cfp(gal_data_t *input, size_t dist,
+                                float thresh, float width, int inplace,
+                                int quiet, size_t *index)
+{
+  gal_data_t  *nbs, *out=NULL;
+  double min, max, check, widthcheck;
+  size_t one=1, flatind=GAL_BLANK_SIZE_T, start=GAL_BLANK_SIZE_T;
+
+  /* Sanity checks. */
+  if(thresh<=0 || width<=0)
+    error(EXIT_FAILURE, 0, "%s: the values to `thresh' (%g) and `width' (%g) "
+          "must be positive", __func__, thresh, width);
+  if(width==0)
+    error(EXIT_FAILURE, 0, "%s: `dist' (%zu) cannot be zero", __func__,
+          dist);
+
+  /* Remove all blanks and sort the dataset. */
+  nbs=gal_statistics_no_blank_sorted(input, inplace);
+
+  /* Find the index where the distribution becomes sufficiently flat. */
+  switch(nbs->type)
+    {
+    case GAL_TYPE_UINT8:   OUTLIER_FLAT_CFP_BYTYPE( uint8_t  ); break;
+    case GAL_TYPE_INT8:    OUTLIER_FLAT_CFP_BYTYPE( int8_t   ); break;
+    case GAL_TYPE_UINT16:  OUTLIER_FLAT_CFP_BYTYPE( uint16_t ); break;
+    case GAL_TYPE_INT16:   OUTLIER_FLAT_CFP_BYTYPE( int16_t  ); break;
+    case GAL_TYPE_UINT32:  OUTLIER_FLAT_CFP_BYTYPE( uint32_t ); break;
+    case GAL_TYPE_INT32:   OUTLIER_FLAT_CFP_BYTYPE( int32_t  ); break;
+    case GAL_TYPE_UINT64:  OUTLIER_FLAT_CFP_BYTYPE( uint64_t ); break;
+    case GAL_TYPE_INT64:   OUTLIER_FLAT_CFP_BYTYPE( int64_t  ); break;
+    case GAL_TYPE_FLOAT32: OUTLIER_FLAT_CFP_BYTYPE( float    ); break;
+    case GAL_TYPE_FLOAT64: OUTLIER_FLAT_CFP_BYTYPE( double   ); break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: type code %d not recognized",
+            __func__, nbs->type);
+    }
+
+  /* Write the output dataset. */
+  if(flatind!=GAL_BLANK_SIZE_T)
+    {
+      out=gal_data_alloc(NULL, input->type, 1, &one, NULL, 0, -1,
+                         NULL, NULL, NULL);
+      memcpy(out->array,
+             gal_pointer_increment(nbs->array, flatind, nbs->type),
+             gal_type_sizeof(nbs->type));
+    }
+
+  /* Clean up and return. */
+  if(nbs!=input) gal_data_free(nbs);
+  if(index) *index=flatind;
+  return out;
+}
