@@ -3,9 +3,9 @@ Type -- Type information and basic operations.
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016, Free Software Foundation, Inc.
+Copyright (C) 2016-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -34,6 +34,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/type.h>
 #include <gnuastro/data.h>
 #include <gnuastro/list.h>
+#include <gnuastro/pointer.h>
 #include <gnuastro-internal/checkset.h>
 
 
@@ -279,7 +280,29 @@ gal_type_max(uint8_t type, void *in)
     case GAL_TYPE_FLOAT32:      *(float *)    in = FLT_MAX;      break;
     case GAL_TYPE_FLOAT64:      *(double *)   in = DBL_MAX;      break;
     default:
-      error(EXIT_FAILURE, 0, "%s: type code %d not recognized", __func__, type);
+      error(EXIT_FAILURE, 0, "%s: type code %d not recognized", __func__,
+            type);
+    }
+}
+
+
+
+
+
+int
+gal_type_is_int(uint8_t type)
+{
+  switch(type)
+    {
+    case GAL_TYPE_UINT8:  return 1;
+    case GAL_TYPE_INT8:   return 1;
+    case GAL_TYPE_UINT16: return 1;
+    case GAL_TYPE_INT16:  return 1;
+    case GAL_TYPE_UINT32: return 1;
+    case GAL_TYPE_INT32:  return 1;
+    case GAL_TYPE_UINT64: return 1;
+    case GAL_TYPE_INT64:  return 1;
+    default:              return 0;
     }
 }
 
@@ -340,7 +363,8 @@ gal_type_bit_string(void *in, size_t size)
 {
   size_t i;
   char *byte=in;
-  char *str=gal_data_malloc_array(GAL_TYPE_UINT8, 8*size+1, __func__, "str");
+  char *str=gal_pointer_allocate(GAL_TYPE_UINT8, 8*size+1, 0, __func__,
+                                 "str");
 
   /* Print the bits into the allocated string. This was inspired from
 
@@ -362,7 +386,10 @@ gal_type_bit_string(void *in, size_t size)
 
 /* Write the contents of memory that `ptr' points to as a string of type
    `type'.*/
-#define TO_STRING(CTYPE, FMT) asprintf(&str, FMT, *(CTYPE *)ptr);
+#define TO_STRING(CTYPE, FMT) {                                         \
+  if( asprintf(&str, FMT, *(CTYPE *)ptr)<0 )                            \
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__); }
+
 char *
 gal_type_to_string(void *ptr, uint8_t type, int quote_if_str_has_space)
 {
@@ -376,11 +403,20 @@ gal_type_to_string(void *ptr, uint8_t type, int quote_if_str_has_space)
       if(quote_if_str_has_space)
         {
           c=*(char **)ptr; while(*c!='\0') if(isspace(*c++)) break;
-          if(*c=='\0') asprintf(&str, "%s",      *(char **)ptr);
-          else         asprintf(&str, "\"%s\" ", *(char **)ptr);
+          if(*c=='\0')
+            {
+              if( asprintf(&str, "%s", *(char **)ptr)<0 )
+                error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+            }
+          else
+            {
+              if( asprintf(&str, "\"%s\" ", *(char **)ptr)<0 )
+                error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+            }
         }
       else
-        asprintf(&str, "%s", *(char **)ptr);
+        if( asprintf(&str, "%s", *(char **)ptr)<0 )
+          error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       break;
 
     case GAL_TYPE_UINT8:   TO_STRING( uint8_t,  "%"PRIu8  );  break;
@@ -437,7 +473,7 @@ gal_type_from_string(void **out, char *string, uint8_t type)
   if( *out==NULL && !gal_type_is_list(type) )
     {
       allocated=1;
-      *out=gal_data_malloc_array(type, 1, __func__, "out");
+      *out=gal_pointer_allocate(type, 1, 0, __func__, "out");
     }
   value=*out;
 
@@ -554,10 +590,17 @@ gal_type_string_to_number(char *string, uint8_t *type)
         }
       else
         {
-          if     (d<=UINT8_MAX)  { u8=d;  ptr=&u8;  *type=GAL_TYPE_UINT8;  }
-          else if(d<=UINT16_MAX) { u16=d; ptr=&u16; *type=GAL_TYPE_UINT16; }
-          else if(d<=UINT32_MAX) { u32=d; ptr=&u32; *type=GAL_TYPE_UINT32; }
-          else                   { u64=d; ptr=&u64; *type=GAL_TYPE_UINT64; }
+          /* Note that the blank values are set to the maximum values in
+             unsigned types. A blank value should be given as a blank
+             string to this function (`GAL_BLANK_STRING'). So, to avoid
+             confusing situations (for example when the user gives 255), if
+             the value is equal to the given maximum of the given type,
+             we'll assign it to a larger type. In other words, we won't be
+             using the `<=MAX', but `<MAX'. */
+          if     (d<UINT8_MAX)  { u8=d;  ptr=&u8;  *type=GAL_TYPE_UINT8;  }
+          else if(d<UINT16_MAX) { u16=d; ptr=&u16; *type=GAL_TYPE_UINT16; }
+          else if(d<UINT32_MAX) { u32=d; ptr=&u32; *type=GAL_TYPE_UINT32; }
+          else                  { u64=d; ptr=&u64; *type=GAL_TYPE_UINT64; }
         }
     }
   else
@@ -597,7 +640,7 @@ gal_type_string_to_number(char *string, uint8_t *type)
     }
 
   /* Allocate a one-element dataset, then copy the number into it. */
-  out=gal_data_malloc_array(*type, 1, __func__, "out");
+  out=gal_pointer_allocate(*type, 1, 0, __func__, "out");
   memcpy(out, ptr, gal_type_sizeof(*type));
   return out;
 }
