@@ -4,8 +4,9 @@
 # (in the Installing gnuastro section).
 #
 # Original author:
-#     Mohammad Akhlaghi <akhlaghi@gnu.org>
+#     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 # Contributing author(s):
+# Copyright (C) 2015-2019 Free Software Foundation, Inc.
 #
 # Copying and distribution of this file, with or without modification,
 # are permitted in any medium without royalty provided the copyright
@@ -19,7 +20,7 @@
 # Preliminaries
 # =============
 #
-# Set the variabels (The executable is in the build tree). Do the
+# Set the variables (The executable is in the build tree). Do the
 # basic checks to see if the executable is made or if the defaults
 # file exists (basicchecks.sh is in the source tree).
 img=psf.jpg
@@ -42,9 +43,9 @@ execname=../bin/$prog/ast$prog
 #     data file failed).
 #
 #   - libjpeg was not present on the system.
-if [ ! -f $execname ] || [ ! -f $img ] || [ "x$haslibjpeg" != "xyes" ]; then
-    exit 77;
-fi
+if [ ! -f $execname           ];then echo "$execname not created.";exit 77;fi
+if [ ! -f $img                ];then echo "$img does not exist.";  exit 77;fi
+if [ "x$haslibjpeg" != "xyes" ];then echo "libjpeg not present.";  exit 77;fi
 
 
 
@@ -53,4 +54,8 @@ fi
 
 # Actual test script
 # ==================
-$execname $img --output=jpegtotxt.txt
+#
+# `check_with_program' can be something like `Valgrind' or an empty
+# string. Such programs will execute the command if present and help in
+# debugging when the developer doesn't have access to the user's system.
+$check_with_program $execname $img --output=jpegtotxt.txt
